# -*- coding: utf-8 -*-
from _common import M2mMapping
from collections import MutableMapping
from util import parse
from copy import deepcopy

__author__ = 'dengjing'


class m2mdict(M2mMapping, MutableMapping):
    def __iter__(self):
        for k, v in self._bmap.items():
            yield k, v

    def __delitem__(self, key):
        del self._fmap[key]

    def items(self):
        return self._fmap.items()

    def keys(self):
        ks = []
        for k, v in self.items():
            ks.append(k)
        return ks

    def __setitem__(self, key, value):
        self._fmap[key] = value

    def getdefault(self, key):
        return self._fmap[key][0]

    def setdefault(self, key, value):
        if isinstance(self._fmap[key], list):
            if value in self._fmap[key]:
                self._fmap[key].insert(0, (self._fmap.pop(self._fmap[key].index(value))))
            else:
                self._fmap[key].insert(0, value)
        else:
            if key not in self.keys():
                self._fmap[key] = [value]
            else:
                raise Exception('the value expect a base.')


if __name__ == '__main__':
    d = m2mdict([("img/x-jpeg", "jpg"), ("img/jpeg", "jpg"), ("img/jpeg", "jepg")])
    print(d.items())
    print(d.getdefault('img/jpeg'))
    # print ~d
    d.setdefault('img/jpeg', 'jpeg')
    print(d.getdefault('img/jpeg'))


# -*- coding: utf-8 -*-
from collections import Mapping
from util import parse, check_args

__author__ = 'dengjing'


class M2mMapping(Mapping):
    """
    有向多对多关系基本模式
    生成形式 mm = M2mMapping([("img/x-jpeg": "jpg"), ("img/jpeg": "jpg")])
    """
    def __init__(self, *args, **kwargs):
        check_args(*args, **kwargs)
        self._fmap = {}
        self._bmap = {}
        self._fmap = parse(args, flag=False)
        self._bmap = parse(args, flag=True)
        inv = object.__new__(self.__class__)
        inv._fmap = self._bmap
        inv._bmap = self._fmap
        inv._inv = self
        self._inv = inv

    def __repr__(self):
        return '%s(%r)' % (self.__class__.__name__, self._fmap)

    def __getitem__(self, key):
        if key:
            return self._fmap[key]

    __str__ = __repr__

    # it will be override
    __len__ = lambda self: len(self._fmap)
    __iter__ = lambda self: iter(self._rawmap)

    def __invert__(self):
        """
        Called when the unary inverse operator (~) is applied.
        """
        return self._inv



# -*- coding: utf-8 -*-
from collections import defaultdict

__author__ = 'dengjing'


iteritems = lambda x: iter(x)


def parse(args, flag=False):
    """
    将参数装换到（k --> set(v)）的形式 d.items()
    [('blue', set([2, 4])), ('red', set([1])), ('yellow', set([1, 3]))]
    """
    args = args[0]
    if isinstance(args, list):
        d = defaultdict(list)
        if not flag:
            for k, v in args:
                d[k].append(v)
        else:
            for k, v in args:
                d[v].append(k)
        return d
    else:
        raise TypeError('expected the argument is list')


def check_args(*args, **kwargs):
    if kwargs:
        raise TypeError('there is no need to get kw')
    if args:
        l = len(args)
        if l != 1:
            raise TypeError('expected at most 1 argument, got %d' % l)
        list_or_err = args[0]
        if isinstance(list_or_err, list):
            tmp = set()
            for (k, v) in list_or_err:
                tmp.add((k, v))
            if len(tmp) != len(list_or_err):
                raise Exception('the argument is more than what is due')
            else:
                pass
        else:
            raise TypeError('expected the argument is list')


def reverse(args):
    if isinstance(args, list):
        for (k, v) in args:
            yield (v, k)


__author__ = 'dengjing'
from setuptools import setup, find_packages

setup(
    name='m2mdict',
    version='0.0.3a',
    keywords=('map', 'dict', 'many to many'),
    description = 'a datastruct of many to many',
    license = 'MIT License',
    install_requires = [],
    author = 'dengjing',
    author_email = 'chinadengjing@gmail.com',
    packages = find_packages(),
    platforms = 'any',
)